"""

"""

from logging import getLogger

from django.contrib.admin import ModelAdmin

LOGGER = getLogger(__name__)


class OktaUserModelAdmin(ModelAdmin):
	"""User admin
	Customized user admin for Okta users.
	"""
	
	fieldsets = (
		('Basic Info', {'fields': ('login', 'firstName', 'lastName', 'email')}),
		('Names', {
			'classes': ('collapse',),
			'fields': ('displayName', 'honorificPrefix', 'middleName', 'honorificSuffix', 'nickName'),
		}),
		('Contact Info', {
			'classes': ('collapse',),
			'fields': ('primaryPhone', 'mobilePhone', 'profileUrl', 'secondEmail'),
		}),
		('Organization', {
			'classes': ('collapse',),
			'fields': ('organization', 'division', 'department', 'title', 'userType', 'employeeNumber', 'costCenter',
					   'managerId', 'manager'),
		}),
		('Addresses', {
			'classes': ('collapse',),
			'fields': ('streetAddress', 'city', 'state', 'zipCode', 'countryCode', 'postalAddress'),
		}),
		('International', {
			'classes': ('collapse',),
			'fields': ('locale', 'preferredLanguage', 'timezone'),
		}),
		('Groups', {
			'classes': ('collapse',),
			'fields': ('groups',),
		}),
		('Permissions', {'fields': ('is_staff', 'is_superuser', 'is_active')}),
	)
	list_display = ('login', 'firstName', 'lastName', 'email', 'is_staff', 'is_superuser', 'is_active')
	list_filter = ('is_active', 'userType', 'organization', 'division', 'department', 'title', 'managerId')
	ordering = ('login',)
	search_fields = ('login', 'firstName', 'lastName', 'email')
	
	def save_model(self, request, obj, form, change):
		"""
		Save model
		"""
		
		if change:
			return super().save_model(request, obj, form, change)
		else:
			return type(obj).objects.create_user(**form.cleaned_data)



#python
"""
This module defines Django signals related to Okta events.
"""

from logging import getLogger

from django.dispatch import Signal

LOGGER = getLogger(__name__)

okta_event_hook = Signal()



#python
"""
Mixins for Django Okta Client.
"""

from json import loads as json_loads
from logging import getLogger

from django.conf import settings
from django.contrib.auth import authenticate, login, logout
from django.urls import reverse

from saml2 import BINDING_HTTP_POST, BINDING_HTTP_REDIRECT
from saml2.client import Saml2Client
from saml2.config import SPConfig as SPConfig_

from .exceptions import SAMLAssertionError
from .signals import okta_event_hook
from .signals import events as okta_events_signals

LOGGER = getLogger(__name__)


class LoginLogoutMixin:
	"""
	Handles user login and logout processes, including SAML assertion parsing.
	"""

	def login_user(self, request):
		"""Login user
		Initiates the SAML authentication process by preparing an authentication request and returning the URL to which the user should be redirected for authentication.

		:param request: the Django request
		:type request: object
		:return: The "Location" header
		:rtype: str
		:raises: RuntimeError if there's no "Location" header in the request
		"""

		LOGGER.debug('Logging in: %s', request)

		request.session['next_url'] = SPConfig.next_url(request)
		LOGGER.debug('Saved "next_url" into session: %s', request.session['next_url'])

		saml_client = Saml2Client(config=SPConfig(request))
		LOGGER.debug('Preparing authentication with: %s', saml_client)
		session_id, request_info = saml_client.prepare_for_authenticate()
		LOGGER.debug('Session id %s includes: %s', session_id, request_info)

		for key, value in request_info['headers']:
			if key == 'Location':
				LOGGER.debug('Found "Location" header: %s', value)
				return value

		raise RuntimeError('The "Location" header was not found')

	def logout_user(self, request):
		"""Logs out the current user.
		If the user is authenticated, they are logged out. Otherwise, a RuntimeError is raised. After logout, the user is redirected to the 'next_url' stored in the session or the default next URL.

		:param request: The Django request object.
		:type request: object
		:return: The URL to redirect to after logout.
		:rtype: str
		:raises RuntimeError: If the user is not authenticated.
		"""
		if request.user.is_authenticated:
			LOGGER.info('Logging out user: %s', request.user)
			logout(request)
		else:
			raise RuntimeError('User is not authenticated')

		next_url = request.session.get('next_url', SPConfig.next_url(request))
		LOGGER.debug('Redirecting after logout: %s', next_url)
		return next_url

	def saml_assertion(self, request):
		"""Handles the SAML assertion process.
		This method is responsible for parsing the SAML response received from the Identity Provider (IdP), authenticating the user based on the SAML assertion, and logging them into the Django application.

		:param request: The Django request object containing the SAML response.
		:type request: object
		:return: The URL to redirect to after successful authentication.
		:rtype: str
		:raises SAMLAssertionError: If there is no SAML response, the response cannot be parsed, or the user identity cannot be extracted.
		:raises RuntimeError: If authentication fails or the OktaBackend is not configured.
		"""

		next_url = request.session.get('next_url', SPConfig.next_url(request))
		saml_client = Saml2Client(config=SPConfig(request))

		response = request.POST.get('SAMLResponse', None)
		if response:
			LOGGER.debug('The ACS received SAML response: %s', response)
		else:
			raise SAMLAssertionError('No POST request to the ACS')

		authn_response = saml_client.parse_authn_request_response(response, BINDING_HTTP_POST)
		if authn_response is None:
			raise SAMLAssertionError(f'Unable to parse SAML response: {response}')
		else:
			LOGGER.debug('Parsed SAML response: %s', authn_response)

		login_id = authn_response.get_subject().text

		user_identity = authn_response.get_identity()
		if user_identity is None:
			raise SAMLAssertionError(f'Malformed SAML response (get_identity failed): {authn_response}')
		else:
			LOGGER.debug('Identity correctly extracted: %s', user_identity)

		saml_values = {key: value[0] if isinstance(value, list) and (len(value) == 1) else value for key, value in user_identity.items() if key not in ['login', 'request']}

		user = authenticate(request, login=login_id, **saml_values)
		if user is None:
			raise RuntimeError('Unable to authenticate. Did you add "okta_client.auth_backends.OktaBackend" to AUTHENTICATION_BACKENDS on your settings.py?')

		LOGGER.info('Logging in "%s"', user)
		login(request, user)

		LOGGER.debug('Redirecting after login to "%s"', next_url)
		return next_url


class OktaEventHookMixin:
	"""
	Handles Okta event hooks, including verification and processing of incoming events.
	"""

	LOCAL_EVENT_HANDLERS = {
	}

	def authenticate_endpoint(self, request):
		"""Authenticates the Okta event hook endpoint.
		This method is called when Okta attempts to verify the event hook endpoint. It extracts the 'x-okta-verification-challenge' header from the request and returns it as part of a dictionary. Okta expects this challenge to be returned to confirm the endpoint's authenticity.

		:param request: The Django request object containing the verification challenge.
		:type request: object
		:return: A dictionary containing the 'verification' challenge string.
		:rtype: dict
		"""

		return {'verification': request.headers.get('x-okta-verification-challenge', '')}

	def handle_events(self, request):
		"""Handles incoming Okta event hook notifications.
		This method parses the JSON payload from the request body, which represents an Okta event hook. It then dispatches this event to registered signal handlers via the `okta_event_hook` signal. It logs the outcome of each handler's execution, noting errors, unexpected return values, or successful completion. Okta does not expect a response body for event hooks, so any return values from handlers are logged as warnings and discarded.

		:param request: The Django request object containing the Okta event hook payload in its body.
		:type request: object
		:return: None
		"""

		event_hook = json_loads(request.body)

		signals_ = {okta_event_hook : {'event_hook': event_hook}}
		results = []

		if ('data' not in event_hook) or ('events' not in event_hook['data']):
			LOGGER.warning('The submitted Okta hook contains no events: %s', event_hook)
		else:
			for event in event_hook['data']['events']:
				signal_name = event['eventType'].replace('.', '_')
				if hasattr(okta_events_signals, signal_name):
					signals_[getattr(okta_events_signals, signal_name)] = {'event': event}
				else:
					LOGGER.warning('Support not implemented for Okta event: %s', event['eventType'])

				if event['eventType'] in self.LOCAL_EVENT_HANDLERS:
					local_handler = self.LOCAL_EVENT_HANDLERS[event['eventType']]
					try:
						local_result = local_handler(event)
					except Exception:
						results.append((local_handler, Exception))
					else:
						results.append((local_handler, local_result))

		for signal_, params in signals_.items():
			results += signal_.send_robust(self.__class__, **params)

		for handler, result in results:
			handler = '.'.join((handler.__module__, handler.__name__))
			if isinstance(result, Exception):
				LOGGER.error('Handler "%s" experienced an error while processing an Okta event hook: %s', handler, result)
			elif result:
				LOGGER.warning('Handler "%s" returned a value while processing an Okta event hook. Okta does not expect an answer, discarding: %s', handler, result)
			else:
				LOGGER.debug('Handler "%s" completed successfully the processing of an Okta event hook.', handler)


class SPConfig:
	"""
	Manages the Service Provider (SP) configuration for SAML authentication.
	"""

	class OktaConfig(dict):
		"""
		Represents the Okta configuration for the Service Provider (SP).
		"""

		def __init__(self, request, django_settings=settings):
			"""Initializes the OktaConfig with request and Django settings.
			This constructor sets up the SAML Service Provider (SP) configuration based on the provided Django settings and the current request. It determines the Assertion Consumer Service (ACS) URL, configures metadata (local or remote), and defines various SAML SP service parameters like endpoints, signing requirements, and name ID format.

			:param request: The Django request object.
			:type request: object
			:param django_settings: The Django settings object, defaults to `settings`.
			"""

			try:
				okta_settings = django_settings.OKTA_CLIENT
			except AttributeError:
				raise ValueError('Missing OKTA_CLIENT section in Django settings')

			super().__init__()

			local_domain_url = okta_settings.get('ASSERTION_DOMAIN_URL', '{}://{}'.format(request.scheme, request.get_host()))
			acs_url = ''.join((local_domain_url, reverse('okta-client:acs')))

			if 'METADATA_LOCAL_FILE_PATH' in okta_settings:
				self['metadata'] = {'local': okta_settings['METADATA_LOCAL_FILE_PATH']}
			else:
				self['metadata'] = {'remote': [{'url': okta_settings['METADATA_AUTO_CONF_URL']}]}

			self['entityid'] = okta_settings.get('ENTITY_ID', acs_url)
			self['service'] = {
				'sp': {
					'endpoints': {
						'assertion_consumer_service': [
							(acs_url, BINDING_HTTP_REDIRECT),
							(acs_url, BINDING_HTTP_POST)
						],
					},
					'allow_unsolicited': True,
					'authn_requests_signed': False,
					'logout_requests_signed': True,
					'want_assertions_signed': True,
					'want_response_signed': False,
				},
			}

			if 'NAME_ID_FORMAT' in okta_settings:
				self['service']['sp']['name_id_format'] = okta_settings['NAME_ID_FORMAT']

	def __new__(cls, request, django_config=settings):
		"""Creates a new SPConfig instance, loading the Okta configuration.
		This method acts as a factory for `SPConfig_` (from `saml2.config`), initializing it with the Okta-specific SAML configuration.

		:param request: The Django request object.
		:param django_config: The Django settings object, defaults to `settings`.
		:return: An initialized `saml2.config.SPConfig` object.
		"""

		sp_config = SPConfig_()
		sp_config.load(cls.OktaConfig(request, django_config))
		sp_config.allow_unknown_attributes = True
		return sp_config

	@classmethod
	def next_url(cls, request, django_config=settings):
		"""Determines the next URL for redirection after a successful login or logout.
		It first checks for a 'next' parameter in the request's GET query. If not found, it falls back to the 'DEFAULT_NEXT_URL' defined in the Okta client settings. If neither is specified, it defaults to the root URL ('/').

		:param request: The Django request object.
		:type request: object
		:param django_config: The Django settings object, defaults to `settings`.
		:type django_config: object
		:return: The URL to redirect to.
		:rtype: str
		"""

		config = cls.OktaConfig(request, django_config)
		return request.GET.get('next', config.get('DEFAULT_NEXT_URL', '/'))


#python
"""
Okta authentication backend for Django.
"""

from logging import getLogger

from django.conf import settings
from django.contrib.auth import get_user_model
from django.contrib.auth.backends import ModelBackend
from django.contrib.auth.models import Group

from rest_framework.authentication import TokenAuthentication as DjangoRESTTokenAuthentication

from .api_client import OktaAPIClient

LOGGER = getLogger(__name__)
UserModel = get_user_model()


class OktaBackend(ModelBackend):
	"""Okta auth backend
	Include Okta related specifics to the authentication process.
	"""

	_api_client = OktaAPIClient()

	def authenticate(self, request, login, **user_details):
		"""Noop check
		Retrieve the user details using the Okta API, check for admin access, and update group membership (creating groups if needed)
		"""

		try:
			self._api_client.okta_api_client
		except Exception as err:
			okta_user = None
			LOGGER.warning('Unable to use the Okta API client: %s', err)
		else:
			try:
				okta_user = self._api_client('get_user', login)
			except RuntimeError:
				LOGGER.error('User not found in Okta: %s', login)
				return None

		try:
			user = UserModel.objects.get(pk=login)
		except UserModel.DoesNotExist:
			LOGGER.debug('Creating new local user: %s', login)
			user = UserModel(login=login) if okta_user is None else UserModel.from_okta_profile(okta_user.profile)
		else:
			if okta_user is not None:
				LOGGER.debug('Updating existing local user: %s', login)
				user.update_from_okta_profile(okta_user.profile)
		if user_details:
			LOGGER.info('Updating user "%s" with values from the SAML assertion: %s', login, user_details)
			user.update(**user_details)
		user.is_active = True
		
		user_groups = []
		if okta_user is not None:
			user_groups = [group.profile.name for group in self._api_client('list_user_groups', okta_user.id)]

			if 'SUPER_USER_GROUPS' in settings.OKTA_CLIENT:
				if frozenset(settings.OKTA_CLIENT['SUPER_USER_GROUPS']) & frozenset(user_groups):
					LOGGER.debug('Found a super user: %s', login)
					user.is_staff = True
					user.is_superuser = True

			if 'STAFF_USER_GROUPS' in settings.OKTA_CLIENT:
				if frozenset(settings.OKTA_CLIENT['STAFF_GROUPS']) & frozenset(user_groups):
					LOGGER.debug('Found a staff user: %s', login)
					user.is_staff = True

		user.save()

		if okta_user is not None:
			LOGGER.debug('Adding user to groups: %s -> %s', login, user_groups)
			for group_name in user_groups:
				try:
					group = Group.objects.get(name=group_name)
				except Group.DoesNotExist:
					LOGGER.debug('Creating local group: %s', group_name)
					group = Group(name=group_name)
					group.save()
				group.user_set.add(user)

			leaving_groups = frozenset([user_group.name for user_group in user.groups.all()]) - frozenset(user_groups)
			if leaving_groups:
				LOGGER.debug('Removing user from groups: %s <- %s', login, list(leaving_groups))
				for removing_from_group in leaving_groups:
					group = Group.objects.get(name=removing_from_group)
					group.user_set.remove(user)

		return user


class DjangoRESTBearerTokenAuthentication(DjangoRESTTokenAuthentication):
	"""Token authentication for REST
	Like the builtin rest_framework.authentication.TokenAuthentication method but using the "Bearer" word instead of the default "Token" (it simplifies the compatibility with Postman)
	"""

	keyword = 'Bearer'


#python
"""Okta Client Models
This module defines the Django models for integrating with Okta user profiles.
"""

from datetime import datetime as DateTime
from logging import getLogger

from django.contrib.auth.models import AbstractBaseUser, PermissionsMixin
from django.core.validators import RegexValidator
from django.db.models import BooleanField, CharField, DateTimeField, EmailField, URLField, fields
from django.utils import timezone as timezone_utils
from django.utils.translation import gettext_lazy as _

from .managers import OktaUserManager

LOGGER = getLogger(__name__)

validator_5_to_100 = RegexValidator(regex = '^.{5,100}$', message = '5 <= value length <= 100')
nullable_5_to_100 = RegexValidator(regex = '(^.{5,100}$)|(ˆ$)', message = '5 <= value length <= 100 (or nothing)')

class AbstractOktaUser(AbstractBaseUser, PermissionsMixin):
	"""Default Okta profile
	Based on the official documentation as of (10/2021) https://developer.okta.com/docs/reference/api/users/#default-profile-properties
	"""
	
	login = CharField(primary_key=True, max_length=100, validators=[validator_5_to_100], verbose_name=_('login'), help_text=_('unique identifier for the user (username)'))
	email = EmailField(blank=False, validators=[validator_5_to_100], verbose_name=_('email'), help_text=_('primary email address of user'))
	secondEmail = EmailField(blank=True, validators=[nullable_5_to_100], verbose_name=_('second email'), help_text=_('secondary email address of user typically used for account recovery'))
	firstName = CharField(blank=False, max_length=50, verbose_name=_('first name'), help_text=_('given name of the user (givenName)'))
	lastName = CharField(blank=False, max_length=50, verbose_name=_('last name'), help_text=_('family name of the user (familyName)'))
	middleName = CharField(blank=True, max_length=50, verbose_name=_('middle name'), help_text=_('middle name(s) of the user'))
	honorificPrefix = CharField(blank=True, max_length=50, verbose_name=_('honorific prefix'), help_text=_('honorific prefix(es) of the user, or title in most Western languages'))
	honorificSuffix = CharField(blank=True, max_length=50, verbose_name=_('honorific suffix'), help_text=_('honorific suffix(es) of the user'))
	title = CharField(blank=True, max_length=100, verbose_name=_('title'), help_text=_('user\'s title, such as "Vice President'))
	displayName = CharField(blank=True, max_length=250, verbose_name=_('display name'), help_text=_('name of the user, suitable for display to end users'))
	nickName = CharField(blank=True, max_length=50, verbose_name=_('nickname'), help_text=_('casual way to address the user in real life'))
	profileUrl = URLField(blank=True, verbose_name=_('profile URL'), help_text=_("url of user's online profile (e.g. a web page)"))
	primaryPhone = CharField(blank=True, max_length=100, verbose_name=_('primary phone'), help_text=_('primary phone number of user such as home number'))
	mobilePhone = CharField(blank=True, max_length=100, verbose_name=_('mobile phone'), help_text=_('mobile phone number of user'))
	streetAddress = CharField(blank=True, max_length=100, verbose_name=_('street address'), help_text=_("full street address component of user's address"))
	city = CharField(blank=True, max_length=100, verbose_name=_('city'), help_text=_("city or locality component of user's address (locality)"))
	state = CharField(blank=True, max_length=100, verbose_name=_('state'), help_text=_("state or region component of user's address (region)"))
	zipCode = CharField(blank=True, max_length=100, verbose_name=_('ZIP code'), help_text=_("zipcode or postal code component of user's address (postalCode)"))
	countryCode = CharField(blank=True, max_length=2, verbose_name=_('country code'), help_text=_("country name component of user's address (country)"))
	postalAddress = CharField(blank=True, max_length=100, verbose_name=_('postal address'), help_text=_("mailing address component of user's address"))
	preferredLanguage = CharField(blank=True, max_length=100, verbose_name=_('preferred language'), help_text=_("user's preferred written or spoken languages"))
	locale = CharField(blank=True, max_length=5, verbose_name=_('locale'), help_text=_("user's default location for purposes of localizing items such as currency, date time format, numerical representations, etc."))
	timezone = CharField(blank=True, max_length=100, verbose_name=_('time zone'), help_text=_("user's time zone"))
	userType = CharField(blank=True, max_length=100, verbose_name=_('user type'), help_text=_('used to describe the organization to user relationship such as "Employee" or "Contractor"'))
	employeeNumber = CharField(blank=True, max_length=100, verbose_name=_('employee number'), help_text=_('organization or company assigned unique identifier for the user'))
	costCenter = CharField(blank=True, max_length=100, verbose_name=_('cost center'), help_text=_('name of a cost center assigned to user'))
	organization = CharField(blank=True, max_length=100, verbose_name=_('organization'), help_text=_("name of user's organization"))
	division = CharField(blank=True, max_length=100, verbose_name=_('division'), help_text=_("name of user's division"))
	department = CharField(blank=True, max_length=100, verbose_name=_('department'), help_text=_("name of user's department"))
	managerId = CharField(blank=True, max_length=100, verbose_name=_('manager ID'), help_text=_("id of a user's manager"))
	manager = CharField(blank=True, max_length=100, verbose_name=_('manager'), help_text=_("displayName of the user's manager"))
	
	is_active = BooleanField(default=True, verbose_name=_("active"), help_text=_('Designates whether this user should be treated as active. \nUnselect this instead of deleting accounts.'))
	is_staff = BooleanField(default=False, verbose_name=_("staff status"), help_text=_("Designates whether the user can log into the admin site."))
	date_joined = DateTimeField(verbose_name=_("date joined"), default=timezone_utils.now)
	
	objects = OktaUserManager()
	
	EMAIL_FIELD = 'email'
	USERNAME_FIELD = 'login'
	REQUIRED_FIELDS = ['email', 'firstName', 'lastName']
	
	class Meta:
		verbose_name = _('okta user')
		verbose_name_plural = _('okta users')
		abstract = True
	
	def __str__(self):
		"""String representation of the Okta user.

		:return: The login (username) of the Okta user.
		:rtype: str
		"""

		return self.login
	
	@classmethod
	def _attributes_from_okta_profile(cls, okta_profile):
		"""Extracts relevant attributes from an Okta profile object.
		This method iterates through the model's fields and attempts to retrieve corresponding values from the provided `okta_profile` object. It handles `DateTimeField` conversion and filters out empty or None values.

		:return: Attributes extracted from the Okta profile, suitable for model instantiation or update.
		:rtype: dict
		"""

		attributes = {}
		for field in cls._meta.fields:
			if hasattr(okta_profile, field.name):
				okta_attr = getattr(okta_profile, field.name)
				if (okta_attr is None) or (isinstance(okta_attr, str) and not len(okta_attr)):
					continue
				if isinstance(field, fields.DateTimeField):
					okta_attr = DateTime.fromisoformat(okta_attr.rstrip('Z'))
				attributes[field.name] = okta_attr
		return attributes

	@classmethod
	def from_okta_profile(cls, okta_profile):
		"""Creates an instance of the Okta user model from an Okta profile object.
		This method uses `_attributes_from_okta_profile` to extract relevant data and then instantiates the model.

		:param okta_profile: An object representing the Okta user profile.
		:type okta_profile: object
		:return: An instance of the Okta user model.
		:rtype: cls
		"""

		return cls(**cls._attributes_from_okta_profile(okta_profile))

	def get_full_name(self):
		"""Returns the user's full name.
		This method prioritizes the `displayName` field. If `displayName` is not set, it constructs the full name by concatenating `honorificPrefix`, `firstName`, `middleName`, `lastName`, and `honorificSuffix`, including only non-empty components.

		:return: The user's full name
		:rtype: str
		"""

		if self.displayName:
			return self.displayName
		else:
			components = ('honorificPrefix', 'firstName', 'middleName', 'lastName', 'honorificSuffix')
			return ' '.join([getattr(self, component) for component in components if getattr(self, component)])

	def get_short_name(self):
		"""Returns the user's short name.
		This method prioritizes the `nickName` field. If `nickName` is not set, it returns the `firstName`.

		:return: The user's short name
		:rtype: str
		"""

		if self.nickName:
			return self.nickName
		else:
			return self.firstName

	def update(self, **updated_values):
		"""Updates user attributes from a dictionary of values.
		This method iterates through the provided keyword arguments and updates the corresponding fields on the user model instance. It only updates fields that exist on the model; any unknown fields will be ignored and a warning will be logged.

		:param updated_values: Keyword arguments where keys are model field names and values are the new values for those fields.
		:return: The updated user instance.
		:rtype: self
		"""
		
		local_fields = [field.name for field in self._meta.fields]
		for key, value in updated_values.items():
			if key in local_fields:
				setattr(self, key, value)
			else:
				LOGGER.warning('Dropping unknown field "%s" in object: %s', key, type(self))
		return self

	def update_from_okta_profile(self, okta_profile):
		"""Updates the user's attributes from an Okta profile object.
		This method extracts relevant attributes from the provided `okta_profile` using `_attributes_from_okta_profile` and then updates the corresponding fields on the current user instance.

		:param okta_profile: An object representing the Okta user profile.
		:type okta_profile: object
		:return: The updated user instance.
		:rtype: self
		"""

		for field_name, field_value in self._attributes_from_okta_profile(okta_profile).items():
			setattr(self, field_name, field_value)
		return self


class OktaUser(AbstractOktaUser):
	"""Django user model
	Alternate user model for Django based on Okta profiles.
	"""
	
	class Meta(AbstractOktaUser.Meta):
		swappable = 'AUTH_USER_MODEL'



# Generated by Django 4.1.7 on 2023-03-02 20:40

import django.core.validators
from django.db import migrations, models
import django.utils.timezone
import okta_client.managers


class Migration(migrations.Migration):

    dependencies = [
        ('okta_client', '0001_initial'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='oktauser',
            options={'verbose_name': 'okta user', 'verbose_name_plural': 'okta users'},
        ),
        migrations.AlterModelManagers(
            name='oktauser',
            managers=[
                ('objects', okta_client.managers.OktaUserManager()),
            ],
        ),
        migrations.AlterField(
            model_name='oktauser',
            name='city',
            field=models.CharField(blank=True, help_text="city or locality component of user's address (locality)", max_length=100, verbose_name='city'),
        ),
        migrations.AlterField(
            model_name='oktauser',
            name='costCenter',
            field=models.CharField(blank=True, help_text='name of a cost center assigned to user', max_length=100, verbose_name='cost center'),
        ),
        migrations.AlterField(
            model_name='oktauser',
            name='countryCode',
            field=models.CharField(blank=True, help_text="country name component of user's address (country)", max_length=2, verbose_name='country code'),
        ),
        migrations.AlterField(
            model_name='oktauser',
            name='date_joined',
            field=models.DateTimeField(default=django.utils.timezone.now, verbose_name='date joined'),
        ),
        migrations.AlterField(
            model_name='oktauser',
            name='department',
            field=models.CharField(blank=True, help_text="name of user's department", max_length=100, verbose_name='department'),
        ),
        migrations.AlterField(
            model_name='oktauser',
            name='displayName',
            field=models.CharField(blank=True, help_text='name of the user, suitable for display to end users', max_length=250, verbose_name='display name'),
        ),
        migrations.AlterField(
            model_name='oktauser',
            name='division',
            field=models.CharField(blank=True, help_text="name of user's division", max_length=100, verbose_name='division'),
        ),
        migrations.AlterField(
            model_name='oktauser',
            name='email',
            field=models.EmailField(help_text='primary email address of user', max_length=254, validators=[django.core.validators.RegexValidator(message='5 <= value length <= 100', regex='^.{5,100}$')], verbose_name='email'),
        ),
        migrations.AlterField(
            model_name='oktauser',
            name='employeeNumber',
            field=models.CharField(blank=True, help_text='organization or company assigned unique identifier for the user', max_length=100, verbose_name='employee number'),
        ),
        migrations.AlterField(
            model_name='oktauser',
            name='firstName',
            field=models.CharField(help_text='given name of the user (givenName)', max_length=50, verbose_name='first name'),
        ),
        migrations.AlterField(
            model_name='oktauser',
            name='honorificPrefix',
            field=models.CharField(blank=True, help_text='honorific prefix(es) of the user, or title in most Western languages', max_length=50, verbose_name='honorific prefix'),
        ),
        migrations.AlterField(
            model_name='oktauser',
            name='honorificSuffix',
            field=models.CharField(blank=True, help_text='honorific suffix(es) of the user', max_length=50, verbose_name='honorific suffix'),
        ),
        migrations.AlterField(
            model_name='oktauser',
            name='lastName',
            field=models.CharField(help_text='family name of the user (familyName)', max_length=50, verbose_name='last name'),
        ),
        migrations.AlterField(
            model_name='oktauser',
            name='locale',
            field=models.CharField(blank=True, help_text="user's default location for purposes of localizing items such as currency, date time format, numerical representations, etc.", max_length=5, verbose_name='locale'),
        ),
        migrations.AlterField(
            model_name='oktauser',
            name='login',
            field=models.CharField(help_text='unique identifier for the user (username)', max_length=100, primary_key=True, serialize=False, validators=[django.core.validators.RegexValidator(message='5 <= value length <= 100', regex='^.{5,100}$')], verbose_name='login'),
        ),
        migrations.AlterField(
            model_name='oktauser',
            name='manager',
            field=models.CharField(blank=True, help_text="displayName of the user's manager", max_length=100, verbose_name='manager'),
        ),
        migrations.AlterField(
            model_name='oktauser',
            name='managerId',
            field=models.CharField(blank=True, help_text="id of a user's manager", max_length=100, verbose_name='manager ID'),
        ),
        migrations.AlterField(
            model_name='oktauser',
            name='middleName',
            field=models.CharField(blank=True, help_text='middle name(s) of the user', max_length=50, verbose_name='middle name'),
        ),
        migrations.AlterField(
            model_name='oktauser',
            name='mobilePhone',
            field=models.CharField(blank=True, help_text='mobile phone number of user', max_length=100, verbose_name='mobile phone'),
        ),
        migrations.AlterField(
            model_name='oktauser',
            name='nickName',
            field=models.CharField(blank=True, help_text='casual way to address the user in real life', max_length=50, verbose_name='nickname'),
        ),
        migrations.AlterField(
            model_name='oktauser',
            name='organization',
            field=models.CharField(blank=True, help_text="name of user's organization", max_length=100, verbose_name='organization'),
        ),
        migrations.AlterField(
            model_name='oktauser',
            name='password',
            field=models.CharField(max_length=128, verbose_name='password'),
        ),
        migrations.AlterField(
            model_name='oktauser',
            name='postalAddress',
            field=models.CharField(blank=True, help_text="mailing address component of user's address", max_length=100, verbose_name='postal address'),
        ),
        migrations.AlterField(
            model_name='oktauser',
            name='preferredLanguage',
            field=models.CharField(blank=True, help_text="user's preferred written or spoken languages", max_length=100, verbose_name='preferred language'),
        ),
        migrations.AlterField(
            model_name='oktauser',
            name='primaryPhone',
            field=models.CharField(blank=True, help_text='primary phone number of user such as home number', max_length=100, verbose_name='primary phone'),
        ),
        migrations.AlterField(
            model_name='oktauser',
            name='profileUrl',
            field=models.URLField(blank=True, help_text="url of user's online profile (e.g. a web page)", verbose_name='profile URL'),
        ),
        migrations.AlterField(
            model_name='oktauser',
            name='secondEmail',
            field=models.EmailField(blank=True, help_text='secondary email address of user typically used for account recovery', max_length=254, validators=[django.core.validators.RegexValidator(message='5 <= value length <= 100 (or nothing)', regex='(^.{5,100}$)|(ˆ$)')], verbose_name='second email'),
        ),
        migrations.AlterField(
            model_name='oktauser',
            name='state',
            field=models.CharField(blank=True, help_text="state or region component of user's address (region)", max_length=100, verbose_name='state'),
        ),
        migrations.AlterField(
            model_name='oktauser',
            name='streetAddress',
            field=models.CharField(blank=True, help_text="full street address component of user's address", max_length=100, verbose_name='street address'),
        ),
        migrations.AlterField(
            model_name='oktauser',
            name='timezone',
            field=models.CharField(blank=True, help_text="user's time zone", max_length=100, verbose_name='time zone'),
        ),
        migrations.AlterField(
            model_name='oktauser',
            name='title',
            field=models.CharField(blank=True, help_text='user\'s title, such as "Vice President', max_length=100, verbose_name='title'),
        ),
        migrations.AlterField(
            model_name='oktauser',
            name='userType',
            field=models.CharField(blank=True, help_text='used to describe the organization to user relationship such as "Employee" or "Contractor"', max_length=100, verbose_name='user type'),
        ),
        migrations.AlterField(
            model_name='oktauser',
            name='zipCode',
            field=models.CharField(blank=True, help_text="zipcode or postal code component of user's address (postalCode)", max_length=100, verbose_name='ZIP code'),
        ),
    ]



#!python3
"""
Custom command to rewrite the okta_client.signals.events file with an updated list from Okta's documentation. It expects a CSV file, which is usually published by Okta.
"""

from csv import reader as csv_reader
from importlib import import_module
from logging import getLogger
from pathlib import Path
from urllib.request import urlopen

from django.core.management.base import BaseCommand, CommandError
from django.template.loader import get_template

DEFAULT_CSV_URL = 'https://developer.okta.com/docs/okta-event-types.csv'
DEFAULT_HOOK_ELIGIBLE_TAG = 'event-hook-eligible'
DEFAULT_HEADERS = ('Event Type', 'Description', 'Tags')
DEFAULT_OKTA_EVENT_MODULE = 'okta_client.signals.events'
LOGGER = getLogger(__name__)
SIGNALS_EVENTS_FILE_TEMPLATE = 'okta-client/signals_events.py-template'

class Command(BaseCommand):
	"""
	The custom command
	"""

	help = "Rewrites the module with the Okta event hook signals"

	def add_arguments(self, parser):
		"""
		Adding some optional user parameters
		"""

		parser.add_argument('--csv-url', default=DEFAULT_CSV_URL, help=f'The URL to the CSV file with all the Okta event types; defaults to {DEFAULT_CSV_URL}')
		parser.add_argument('--hook-eligible-tag', default=DEFAULT_HOOK_ELIGIBLE_TAG, help=f'The tag to filter event types by; defaults to "{DEFAULT_HOOK_ELIGIBLE_TAG}"')
		parser.add_argument('--event-type-header', default=DEFAULT_HEADERS[0], help=f'The tag to filter event types by; defaults to "{DEFAULT_HEADERS[0]}"')
		parser.add_argument('--description-header', default=DEFAULT_HEADERS[1], help=f'The tag to filter event types by; defaults to "{DEFAULT_HEADERS[1]}"')
		parser.add_argument('--tags-header', default=DEFAULT_HEADERS[2], help=f'The tag to filter event types by; defaults to "{DEFAULT_HEADERS[2]}"')
		parser.add_argument('--target-module', default=DEFAULT_OKTA_EVENT_MODULE, help=f'The module whose file will be rewritten with the result; defaults to "{DEFAULT_OKTA_EVENT_MODULE}"')

	def handle(self, *args, **options):
		"""Actual command behavior
		It performs some steps to accomplish its goals:
		1. retrieves the CSV file from the supplied URL
		2. uses Python's builtin csv parser to ingest it
		3. creates a map of columns based on the header (1st line of the file) and the supplied options
		4. filter rows based on the "--tags-header" and "--hook-eligible-tag" (the "tags" column should be a comma separated list)
		5. builds a dict with the filtered rows using "--event-type-header" column (with the "." replaced by "_") for the keys and "--description-header" column for the values
		6. uses the SIGNALS_EVENTS_FILE_TEMPLATE with the dict from #5 to generate the new file content
		7. Writes the content to the file in events.__file__
		"""

		try:
			with urlopen(options['csv_url']) as response:
				csv_content = csv_reader((line_.decode() for line_ in response.readlines()))
		except Exception:
			raise CommandError(f"Couldn't load the Okta event types CSV file correctly: {options['csv_url']}")

		csv_header, hookable_event_types = None, {}
		column_map = [None, None, None]
		for row in csv_content:
			if csv_header is None:
				for column_number in range(len(row)):
					if row[column_number] == options['tags_header']:
						column_map[0] = column_number
					elif row[column_number] == options['event_type_header']:
						column_map[1] = column_number
					elif row[column_number] == options['description_header']:
						column_map[2] = column_number
				csv_header = row
				if None in column_map:
					expected_haders = (options['tags_header'], options['event_type_header'], options['description_header'])
					raise CommandError(f"Missing requested columns: {[expected_haders[header_index] for header_index in range(len(expected_haders)) if column_map[header_index] is None]}")
				continue

			current_tags = [tag.strip() for tag in row[column_map[0]].split(',')]
			if options['hook_eligible_tag'] in current_tags:
				hookable_event_types[row[column_map[1]].replace('.','_')] = row[column_map[2]]

		try:
			template = get_template(SIGNALS_EVENTS_FILE_TEMPLATE)
			content = template.render({'signals': hookable_event_types})
		except Exception:
			raise CommandError(f"Couldn't render the template: {SIGNALS_EVENTS_FILE_TEMPLATE}")
		
		try:
			target_module = import_module(options['target_module'])
		except ImportError:
			raise CommandError(f'Not a valid output module: {options['target_module']}')
		
		target_file = Path(target_module.__file__)
		try:
			target_file.write_text(content)
		except Exception:
			raise CommandError(f"Couldn't write to the target file: {target_file}")

		self.stdout.write(self.style.SUCCESS(f'Successfully rewritten OKTA event signals. Total : {len(hookable_event_types)}'))



# Generated by Django 3.2.8 on 2021-10-05 19:28

import django.core.validators
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('auth', '0012_alter_user_first_name_max_length'),
    ]

    operations = [
        migrations.CreateModel(
            name='OktaUser',
            fields=[
                ('last_login', models.DateTimeField(blank=True, null=True, verbose_name='last login')),
                ('is_superuser', models.BooleanField(default=False, help_text='Designates that this user has all permissions without explicitly assigning them.', verbose_name='superuser status')),
                ('login', models.CharField(help_text='unique identifier for the user (username)', max_length=100, primary_key=True, serialize=False, validators=[django.core.validators.RegexValidator(message='5 <= value length <= 100', regex='^.{5,100}$')])),
                ('email', models.EmailField(help_text='primary email address of user', max_length=254, validators=[django.core.validators.RegexValidator(message='5 <= value length <= 100', regex='^.{5,100}$')])),
                ('secondEmail', models.EmailField(blank=True, help_text='secondary email address of user typically used for account recovery', max_length=254, validators=[django.core.validators.RegexValidator(message='5 <= value length <= 100 (or nothing)', regex='(^.{5,100}$)|(ˆ$)')])),
                ('firstName', models.CharField(help_text='given name of the user (givenName)', max_length=50)),
                ('lastName', models.CharField(help_text='family name of the user (familyName)', max_length=50)),
                ('middleName', models.CharField(blank=True, help_text='middle name(s) of the user', max_length=50)),
                ('honorificPrefix', models.CharField(blank=True, help_text='honorific prefix(es) of the user, or title in most Western languages', max_length=50)),
                ('honorificSuffix', models.CharField(blank=True, help_text='honorific suffix(es) of the user', max_length=50)),
                ('title', models.CharField(blank=True, help_text='user\'s title, such as "Vice President', max_length=100)),
                ('displayName', models.CharField(blank=True, help_text='name of the user, suitable for display to end users', max_length=250)),
                ('nickName', models.CharField(blank=True, help_text='casual way to address the user in real life', max_length=50)),
                ('profileUrl', models.URLField(blank=True, help_text="url of user's online profile (e.g. a web page)")),
                ('primaryPhone', models.CharField(blank=True, help_text='primary phone number of user such as home number', max_length=100)),
                ('mobilePhone', models.CharField(blank=True, help_text='mobile phone number of user', max_length=100)),
                ('streetAddress', models.CharField(blank=True, help_text="full street address component of user's address", max_length=100)),
                ('city', models.CharField(blank=True, help_text="city or locality component of user's address (locality)", max_length=100)),
                ('state', models.CharField(blank=True, help_text="state or region component of user's address (region)", max_length=100)),
                ('zipCode', models.CharField(blank=True, help_text="zipcode or postal code component of user's address (postalCode)", max_length=100)),
                ('countryCode', models.CharField(blank=True, help_text="country name component of user's address (country)", max_length=2)),
                ('postalAddress', models.CharField(blank=True, help_text="mailing address component of user's address", max_length=100)),
                ('preferredLanguage', models.CharField(blank=True, help_text="user's preferred written or spoken languages", max_length=100)),
                ('locale', models.CharField(blank=True, help_text="user's default location for purposes of localizing items such as currency, date time format, numerical representations, etc.", max_length=5)),
                ('timezone', models.CharField(blank=True, help_text="user's time zone", max_length=100)),
                ('userType', models.CharField(blank=True, help_text='used to describe the organization to user relationship such as "Employee" or "Contractor"', max_length=100)),
                ('employeeNumber', models.CharField(blank=True, help_text='organization or company assigned unique identifier for the user', max_length=100)),
                ('costCenter', models.CharField(blank=True, help_text='name of a cost center assigned to user', max_length=100)),
                ('organization', models.CharField(blank=True, help_text="name of user's organization", max_length=100)),
                ('division', models.CharField(blank=True, help_text="name of user's division", max_length=100)),
                ('department', models.CharField(blank=True, help_text="name of user's department", max_length=100)),
                ('managerId', models.CharField(blank=True, help_text="id of a user's manager", max_length=100)),
                ('manager', models.CharField(blank=True, help_text="displayName of the user's manager", max_length=100)),
                ('password', models.CharField(blank=True, help_text="user's password", max_length=128)),
                ('is_staff', models.BooleanField(default=False, help_text='Designates whether the user can log into this admin site.', verbose_name='staff status')),
                ('is_active', models.BooleanField(default=True, help_text='Designates whether this user should be treated as active. \nUnselect this instead of deleting accounts.', verbose_name='active')),
                ('date_joined', models.DateTimeField(auto_now_add=True)),
                ('groups', models.ManyToManyField(blank=True, help_text='The groups this user belongs to. A user will get all permissions granted to each of their groups.', related_name='user_set', related_query_name='user', to='auth.Group', verbose_name='groups')),
                ('user_permissions', models.ManyToManyField(blank=True, help_text='Specific permissions for this user.', related_name='user_set', related_query_name='user', to='auth.Permission', verbose_name='user permissions')),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



#python
"""
This module defines Django signals related to Okta events.

It contains the event types that are eligible for use with an event hook only.
Ref: https://developer.okta.com/docs/reference/api/event-types/

---------------------------------------------------
- WARNING - This file is automatically generated! -
---------------------------------------------------
Permanent changes should be applied on the template, located at "okta_client/templates/okta-client/signal_events.py-template"
"""

from logging import getLogger

from django.dispatch import Signal

LOGGER = getLogger(__name__)

access_request_cancel = Signal()
access_request_cancel.__doc__ = """
Access request canceled. Can be used to audit access to an Okta resource or to trigger downstream automation that depends on request cancellation. Okta Identity Governance API can be used to get more details about the canceled request.
"""

access_request_condition_activate = Signal()
access_request_condition_activate.__doc__ = """
Access request condition activated. Can be used to audit access request condition to an Okta resource or to trigger downstream automation that depends on access request condition activation. Okta Identity Governance API can be used to get more details about the activated access request condition.
"""

access_request_condition_create = Signal()
access_request_condition_create.__doc__ = """
Access request condition created. Can be used to audit access request condition to an Okta resource or to trigger downstream automation that depends on access request condition creation. Okta Identity Governance API can be used to get more details about the created access request condition.
"""

access_request_condition_deactivate = Signal()
access_request_condition_deactivate.__doc__ = """
Access request condition deactivated. Can be used to audit access request condition to an Okta resource or to trigger downstream automation that depends on access request condition deactivation. Okta Identity Governance API can be used to get more details about the deactivated access request condition.
"""

access_request_condition_delete = Signal()
access_request_condition_delete.__doc__ = """
Access request condition deleted. Can be used to audit access request condition to an Okta resource or to trigger downstream automation that depends on access request condition deletion. Okta Identity Governance API can be used to get more details about the deleted access request condition.
"""

access_request_condition_invalidate = Signal()
access_request_condition_invalidate.__doc__ = """
Access request condition invalidated. Can be used to audit access request condition to an Okta resource or to trigger downstream automation that depends on access request condition invalidation. Okta Identity Governance API can be used to get more details about the invalidated access request condition.
"""

access_request_condition_update = Signal()
access_request_condition_update.__doc__ = """
Access request condition updated. Can be used to audit access request condition to an Okta resource or to trigger downstream automation that depends on access request condition update. Okta Identity Governance API can be used to get more details about the updated access request condition.
"""

access_request_create = Signal()
access_request_create.__doc__ = """
Access request created. Can be used to audit access to an Okta resource or to trigger downstream automation that depends on request creation. Okta Identity Governance API can be used to get more details about the created request.
"""

access_request_expire = Signal()
access_request_expire.__doc__ = """
Access request expired. Can be used to audit access to an Okta resource or to trigger downstream automation that depends on request expiration. Okta Identity Governance API can be used to get more details about the expired request.
"""

access_request_reject = Signal()
access_request_reject.__doc__ = """
Access request rejected. Can be used to audit access to an Okta resource or to trigger downstream automation that depends on request rejection. Okta Identity Governance API can be used to get more details about the rejected request.
"""

access_request_resolve = Signal()
access_request_resolve.__doc__ = """
Access request resolved. Can be used to audit access to an Okta resource or to trigger downstream automation that depends on request resolution. Okta Identity Governance API can be used to get more details about the resolved request.
"""

access_request_sequence_create = Signal()
access_request_sequence_create.__doc__ = """
Access request sequence created. Can be used to audit the approval sequence and when it was created and what was defined within the sequence to verify the approvals required. Okta Identity Governance API can be used to get more details about the created access request sequence.
"""

access_request_sequence_delete = Signal()
access_request_sequence_delete.__doc__ = """
Access request sequence deleted. Can be used to audit the approval sequence and when it was deleted and what was defined within the sequence to verify the approvals required. Okta Identity Governance API can be used to get more details about the deleted access request sequence.
"""

access_request_sequence_update = Signal()
access_request_sequence_update.__doc__ = """
Access request sequence updated. Can be used to audit the approval sequence and when it was updated and what was defined within the sequence to verify the approvals required. Okta Identity Governance API can be used to get more details about the updated access request sequence.
"""

access_request_settings_update = Signal()
access_request_settings_update.__doc__ = """
Access request settings updated. Can be used to audit access request settings to an Okta resource or to trigger downstream automation that depends on access request settings update. Okta Identity Governance API can be used to get more details about the updated access request settings.
"""

app_access_request_approver_approve = Signal()
app_access_request_approver_approve.__doc__ = """
Request to access an app was approved by an administrator-defined approver.
"""

app_access_request_approver_deny = Signal()
app_access_request_approver_deny.__doc__ = """
Request to access an app was denied by an administrator-defined approver.
"""

app_access_request_delete = Signal()
app_access_request_delete.__doc__ = """
Request to access an app was deleted by an administrator.
"""

app_access_request_deny = Signal()
app_access_request_deny.__doc__ = """
Request to access an app was denied after at least one approver denied the request.
"""

app_access_request_expire = Signal()
app_access_request_expire.__doc__ = """
Request to access an app expired by the system due to lack of approver action.
"""

app_access_request_grant = Signal()
app_access_request_grant.__doc__ = """
Request to access an app was granted after all approvers approved the request.
"""

app_access_request_request = Signal()
app_access_request_request.__doc__ = """
Request to access an app was performed by a user.
"""

app_oauth2_as_consent_grant = Signal()
app_oauth2_as_consent_grant.__doc__ = """
User granted consent to app.
"""

app_oauth2_as_consent_revoke = Signal()
app_oauth2_as_consent_revoke.__doc__ = """
Consent revoked.
"""

app_oauth2_as_consent_revoke_implicit_as = Signal()
app_oauth2_as_consent_revoke_implicit_as.__doc__ = """
All consent revoked for authorization server.
"""

app_oauth2_as_consent_revoke_implicit_client = Signal()
app_oauth2_as_consent_revoke_implicit_client.__doc__ = """
All consent revoked for client.
"""

app_oauth2_as_consent_revoke_implicit_scope = Signal()
app_oauth2_as_consent_revoke_implicit_scope.__doc__ = """
All consent revoked for scope.
"""

app_oauth2_as_consent_revoke_implicit_user = Signal()
app_oauth2_as_consent_revoke_implicit_user.__doc__ = """
Consent for all scopes revoked for user.
"""

app_oauth2_as_consent_revoke_user = Signal()
app_oauth2_as_consent_revoke_user.__doc__ = """
All consent revoked for user.
"""

app_oauth2_as_consent_revoke_user_client = Signal()
app_oauth2_as_consent_revoke_user_client.__doc__ = """
User consent revoked for client.
"""

app_oauth2_client_privilege_grant = Signal()
app_oauth2_client_privilege_grant.__doc__ = """
An OAuth 2.0 client app's admin privileges changed. This can be used to audit the provisioning of admin privileges for OAuth 2.0 client apps. When fired, this event contains information about the type of admin privileges the OAuth 2.0 client app currently has.  Related events include: APP_OAUTH2_CLIENT_PRIVILEGE_REVOKE.
"""

app_oauth2_client_privilege_revoke = Signal()
app_oauth2_client_privilege_revoke.__doc__ = """
All privileges for OAuth 2.0 client app were revoked. This can be used to audit the deprovisioning of admin privileges from OAuth 2.0 client apps. When fired, this event indicates the OAuth 2.0 client app has no more admin privileges. All of OAuth 2.0 client app's privileges were revoked.  Related events include: APP_OAUTH2_CLIENT_PRIVILEGE_GRANT.
"""

app_oauth2_trusted_server_add = Signal()
app_oauth2_trusted_server_add.__doc__ = """
Trusted authorization server is added. Administrators can use this event to debug and audit trusted authorization server operations. When fired, this event contains the authorization server IDs of the servers involved.
"""

app_oauth2_trusted_server_delete = Signal()
app_oauth2_trusted_server_delete.__doc__ = """
Trusted authorization server is removed. Administrators can use this event to debug and audit trusted authorization server operations. When fired, this event contains the authorization server IDs of the servers involved.
"""

application_lifecycle_activate = Signal()
application_lifecycle_activate.__doc__ = """
Activate application.
"""

application_lifecycle_create = Signal()
application_lifecycle_create.__doc__ = """
Create application.
"""

application_lifecycle_deactivate = Signal()
application_lifecycle_deactivate.__doc__ = """
Deactivate application.
"""

application_lifecycle_delete = Signal()
application_lifecycle_delete.__doc__ = """
Delete application.
"""

application_lifecycle_update = Signal()
application_lifecycle_update.__doc__ = """
Update application.
"""

application_policy_sign_on_deny_access = Signal()
application_policy_sign_on_deny_access.__doc__ = """
Deny user access due to app sign on policy. When fired due to app assurance being evaluated as unsatisfiable (the policy requirements could not be satisfied by the users' current set of available authenticator enrollments), this event contains information about the user and the app that the user is trying to authenticate into.
"""

application_provision_group_push_mapping_update_or_delete_failed_with_error = Signal()
application_provision_group_push_mapping_update_or_delete_failed_with_error.__doc__ = """
Group push mapping change failed and cannot be retried. Can be used to identify group push mapping errors which may require admin intervention to address. Unlike the similarly named event, application.provision.group_push.mapping.update.or.delete.failed, when this event is fired the corresponding action that triggered it will not be retried by Okta and may indicate a configuration problem. For example, invalid authorization credentials with the target application due to an expired password or invalid access token.
"""

application_provision_user_sync = Signal()
application_provision_user_sync.__doc__ = """
Sync user in external application.
"""

application_user_membership_add = Signal()
application_user_membership_add.__doc__ = """
Add user to application membership.
"""

application_user_membership_change_password = Signal()
application_user_membership_change_password.__doc__ = """
Change application password for user.
"""

application_user_membership_remove = Signal()
application_user_membership_remove.__doc__ = """
Remove user's application membership.
"""

application_user_membership_update = Signal()
application_user_membership_update.__doc__ = """
Updated user application property.
"""

certification_campaign_close = Signal()
certification_campaign_close.__doc__ = """
Triggered when a campaign is closed. This event can be used by admins to audit Access Certification Review activity to understand when a Campaign Instance has transitioned into the closed state. This event is triggered when a Campaign is closed either by an admin or on the configured campaign end date.
"""

certification_campaign_context_update = Signal()
certification_campaign_context_update.__doc__ = """
Triggered when the customizable context settings are updated at org level. Can be used to audit customizable context settings changes in the system log as well as reports. This is triggered when the customizable context settings are updated at the org level.
"""

certification_campaign_item_decide = Signal()
certification_campaign_item_decide.__doc__ = """
Triggered when a decision on the access to a resource is made. Can be used to audit the decision activity related to an item in a certification campaign, such as the access of a user to an application. The outcome.result field will be SUCCESS for a decision to approve or revoke and will be SKIPPED for a decision to delegate. This is triggered when a reviewer makes a decision on a campaign item, or at the end of a campaign if an item has not been reviewed. The result of the decision is included in the debugData (APPROVE, REVOKE, DELEGATE, NORESPONSE).
"""

certification_campaign_item_remediate = Signal()
certification_campaign_item_remediate.__doc__ = """
Triggered when the remediation is performed on the campaign item. Can be used to audit remediation activity in the system log as well as reports. This is triggered when the remediation is acted upon by reviewer.
"""

certification_campaign_launch = Signal()
certification_campaign_launch.__doc__ = """
Triggered when a campaign is launched. This event can be used by admins to audit Access Certification Review activity to understand when a Campaign Instance has transitioned into the Active state. This event is triggered when a Campaign starts and moves from scheduled to active.
"""

device_enrollment_create = Signal()
device_enrollment_create.__doc__ = """
Enroll new device. This can be used by any admin to monitor when a new device is registered successfully for Okta Verify. The user must have below the max allowed devices and a valid device status (not suspended or deactivated).The targets field contains key details of the enrolled device including name, status, serialNumber, imei, meid, osVersion, osPlatform. which may be useful for identifying the device, tracking  which device platforms and OS versions that enrolled in Okta Device Authenticator.
"""

device_lifecycle_activate = Signal()
device_lifecycle_activate.__doc__ = """
Activate device. You can use the event to audit device status change. When triggered, the device can be suspended or deactivated. Also, a user can access protected resources from an active device if permitted by the App Sign-On policies applied to the resources.
"""

device_lifecycle_deactivate = Signal()
device_lifecycle_deactivate.__doc__ = """
Deactivate device. You can use the event to audit device status change. When a device is deactivated, it can not be associated with any Okta Verify factor in the future.
"""

device_lifecycle_delete = Signal()
device_lifecycle_delete.__doc__ = """
Delete device. You can use the event to audit device status change. When triggered, the device no longer appears in the Admin Console.
"""

device_lifecycle_suspend = Signal()
device_lifecycle_suspend.__doc__ = """
Suspend device. You can use the event to audit device status change. When triggered, access to the device is temporarily paused for users such as contractors or employees who take a leave of absence. Only active devices can be suspended. If a device suspension fails, the cause may be that the device was not active and therefore cannot be suspended.
"""

device_lifecycle_unsuspend = Signal()
device_lifecycle_unsuspend.__doc__ = """
Unsuspend device. You can use the event to audit device status change. When triggered, all Okta Verify factors associated with the device are unsuspended, and users can access protected resources from the device.
"""

device_user_add = Signal()
device_user_add.__doc__ = """
Add device to user. You can use the event to audit device user association activity. The event is triggered when a user adds a new account in Okta Verify.
"""

device_user_remove = Signal()
device_user_remove.__doc__ = """
Remove device from user. You can use the event to audit device user association activity. The device remains in the Universal Directory after the user is removed.
"""

group_application_assignment_add = Signal()
group_application_assignment_add.__doc__ = """
Add assigned application to group.
"""

group_application_assignment_remove = Signal()
group_application_assignment_remove.__doc__ = """
Remove assigned application from group.
"""

group_application_assignment_update = Signal()
group_application_assignment_update.__doc__ = """
Update assigned application in group.
"""

group_lifecycle_create = Signal()
group_lifecycle_create.__doc__ = """
Create Okta group. This can be used to make sure an Okta group is successfully created. Event fired when an Okta group is successfully created.
"""

group_lifecycle_delete = Signal()
group_lifecycle_delete.__doc__ = """
Delete Okta group. This can be used to make sure an Okta group is successfully deleted. Event fired when an Okta group is successfully deleted.
"""

group_privilege_grant = Signal()
group_privilege_grant.__doc__ = """
Group's admin privilege granted. This can be used to audit the provisioning of admin privileges for groups. When fired, this event contains information about the type of admin privileges the group currently has, and what entity sources the group. The group granted privileges can be an Okta sourced group, and AD-sourced group, or an LDAP-sourced group Related events include: GROUP_PRIVILEGE_REVOKE.
"""

group_privilege_revoke = Signal()
group_privilege_revoke.__doc__ = """
Group's admin privilege revoked. This can be used to audit the deprovisioning of admin privileges from groups. When fired, this event indicates the group has no more admin privileges. All of group's privileges were revoked.  Related events include: GROUP_PRIVILEGE_GRANT.
"""

group_profile_update = Signal()
group_profile_update.__doc__ = """
Okta group profile updated. Events of this type can be used by an IT administrator who wants to trigger an Okta Workflow to provision groups into downstream systems. The utility of the Event type is for Provisioning use cases to downstream systems.A classic example of this is a customer who uses Okta for Office 365 LCM, and wants to push a distribution list from Okta to Office 365.
"""

group_user_membership_add = Signal()
group_user_membership_add.__doc__ = """
Add user to group membership.
"""

group_user_membership_remove = Signal()
group_user_membership_remove.__doc__ = """
Remove user from group membership.
"""

iam_policy_configuration_update = Signal()
iam_policy_configuration_update.__doc__ = """
IAM policy configuration update. Use this event to track and audit updates to IAM policy configuration for the org. This event contains information about the IAM policy configuration updates for an org.
"""

iam_resourceset_bindings_add = Signal()
iam_resourceset_bindings_add.__doc__ = """
Admin role assignment is created. This event can be used to track and audit when a new admin role assignment is created. When fired this event contains information about the new user or group admin assignments for roles associated with the resource set.
"""

iam_resourceset_bindings_delete = Signal()
iam_resourceset_bindings_delete.__doc__ = """
Admin assignment is deleted. This event can be used to track and audit when an admin role assignment is deleted. When fired this event contains information about the deleted user or group admin assignments for roles associated with the resource set.
"""

iam_resourceset_create = Signal()
iam_resourceset_create.__doc__ = """
Resource set is created. This event can be used to track and audit when a resource set is created. When fired this event contains information about the resources contained in the resource set that is created.
"""

iam_resourceset_delete = Signal()
iam_resourceset_delete.__doc__ = """
Resource set is deleted. This event can be used to track and audit when a resource set is deleted. When fired this event contains information about the resources contained in the resource set that is deleted.
"""

iam_resourceset_resources_add = Signal()
iam_resourceset_resources_add.__doc__ = """
Resources are added to a resource set. This event can be used to audit the resources added to a resource set. When fired this event contains information about the resources added to the resource set.
"""

iam_resourceset_resources_delete = Signal()
iam_resourceset_resources_delete.__doc__ = """
Resources are deleted from a resource set. This event can be used to audit the resources deleted from a resource set. When fired this event contains information about the resources deleted from the resource set.
"""

iam_resourceset_resources_update = Signal()
iam_resourceset_resources_update.__doc__ = """
Resources updated in a resource set. Used this event to audit resources updated in a resource set. This event contains information about resources that were updated in a resource set.
"""

iam_resourceset_update = Signal()
iam_resourceset_update.__doc__ = """
Resource set update. Use this event to track and audit when a resource set was updated. This event contains information about the updated name and description of the resource set.
"""

iam_role_create = Signal()
iam_role_create.__doc__ = """
Custom admin role is created. This event can be used to track and audit when a custom admin role is created. When fired this event contains information about the permissions contained in the role that is created.
"""

iam_role_delete = Signal()
iam_role_delete.__doc__ = """
Custom admin role is deleted. This event can be used to track and audit when a custom admin role is deleted. When fired this event contains information about the permissions contained in the role that is deleted.
"""

iam_role_permission_conditions_add = Signal()
iam_role_permission_conditions_add.__doc__ = """
Conditions added to a permission in Okta. Use this event to evaluate impact on admin privileges as their scope might be impacted. This event is triggered when a condition is added to a role-based permission in Okta. A condition on a permission allows super admins to implement finer grained authorizations for stricter security postures. The event can be  accompanied with other events for permissions such as iam.role.permissions.add.
"""

iam_role_permission_conditions_delete = Signal()
iam_role_permission_conditions_delete.__doc__ = """
Conditions deleted from a permission in Okta. Use this event to evaluate impact on admin privileges as their scope might be impacted. This event is triggered when a condition is deleted from a role-based permission in Okta. A condition on a permission allows super admins to implement finer grained authorizations for stricter security postures. The event can be  accompanied with other events for permissions such as iam.role.permissions.delete.
"""

iam_role_permissions_add = Signal()
iam_role_permissions_add.__doc__ = """
Permissions are added to a custom admin role. This event can be used to audit the permissions added to a custom admin role. When fired this event contains information about the permissions added to the role.
"""

iam_role_permissions_delete = Signal()
iam_role_permissions_delete.__doc__ = """
Permissions are deleted from a custom admin role. This event can be used to audit the permissions deleted from a custom admin role. When fired this event contains information about the permissions deleted from the role.
"""

iam_role_update = Signal()
iam_role_update.__doc__ = """
Custom admin role update. Use this event to track and audit when a custom admin role was updated. This event contains information about the updated name and description of the role.
"""

inline_hook_executed = Signal()
inline_hook_executed.__doc__ = """
Triggered when an inline hook has been executed. Used to notify admins about the outcome of execution of an inline hook. Note that the event is fired when the execution is unsuccessful.
"""

policy_auth_reevaluate_fail = Signal()
policy_auth_reevaluate_fail.__doc__ = """
Auth policy re-evaluation has occurred and has resulted in a policy violation. Can be used to identify which user, apps, and session were involved in a policy violation event. Event fired when continuing access evaluation results in failure.
"""

policy_lifecycle_activate = Signal()
policy_lifecycle_activate.__doc__ = """
Activate policy.
"""

policy_lifecycle_deactivate = Signal()
policy_lifecycle_deactivate.__doc__ = """
Deactivate policy.
"""

policy_lifecycle_update = Signal()
policy_lifecycle_update.__doc__ = """
Update policy.
"""

policy_rule_activate = Signal()
policy_rule_activate.__doc__ = """
Activate policy rule.
"""

policy_rule_add = Signal()
policy_rule_add.__doc__ = """
Add policy rule.
"""

policy_rule_deactivate = Signal()
policy_rule_deactivate.__doc__ = """
Deactivate policy rule.
"""

policy_rule_delete = Signal()
policy_rule_delete.__doc__ = """
Delete policy rule.
"""

policy_rule_update = Signal()
policy_rule_update.__doc__ = """
Update policy rule.
"""

security_authenticator_lifecycle_activate = Signal()
security_authenticator_lifecycle_activate.__doc__ = """
Fired when an admin activates an authenticator for the org. This event can be used to identify who activated an authenticator and which authenticator was activated. When fired, this event contains information about the authenticator type that was activated and the actor who activated the authenticator. Authenticator activation occurs when an authenticator is added. Related events include security.authenticator.lifecycle.deactivate.
"""

security_authenticator_lifecycle_create = Signal()
security_authenticator_lifecycle_create.__doc__ = """
Fired when an admin creates an authenticator for the org. This event can be used to identify who created an authenticator and which authenticator was created. The actor specifies the user that created the authenticator and the target specifies the authenticator name and the id. This event could also contain some authenticator specific information. Authenticator creation occurs when an authenticator is added. Related events include security.authenticator.lifecycle.update.
"""

security_authenticator_lifecycle_deactivate = Signal()
security_authenticator_lifecycle_deactivate.__doc__ = """
Fired when an admin deactivates an authenticator for the org. This event can be used to identify who deactivated an authenticator and which authenticator was deactivated. When fired, this event contains information about the authenticator type that was deactivated and the actor who deactivated the authenticator. Authenticator deactivation occurs when an authenticator is removed. Related events include security.authenticator.lifecycle.activate.
"""

security_authenticator_lifecycle_update = Signal()
security_authenticator_lifecycle_update.__doc__ = """
Fired when an admin updates an authenticator in the org. This event can be used to identify who updated an authenticator and which authenticator was updated. The actor specifies the user that updated the authenticator and the target specifies the authenticator name and the ID. There may be a second target with details of any authenticator method updates. This event could also contain authenticator specific information. Authenticator update occurs when an authenticator is edited. Related events include security.authenticator.lifecycle.create.
"""

security_breached_credential_detected = Signal()
security_breached_credential_detected.__doc__ = """
A credential, such as a password, which is associated with a known breach was used during an authentication flow. Used to identify users for whom credential rotation or other risk mitigation is necessary. The actor is the user with the breached credential. For Identity Engine, a target will indicate the specific credential associated with the breach. The outcome for this event will always be SUCCESS with a severity level of WARN. If breached credential protection is enabled, auser.session.clear will also be fired. These two events can be correlated by the Request ID.
"""

security_trusted_origin_activate = Signal()
security_trusted_origin_activate.__doc__ = """
A trusted origin is activated. When an event is emitted upon the activation of a trusted origin, customers can monitor these events and take remedial action. Event is triggered when a trusted origin is activated.
"""

security_trusted_origin_create = Signal()
security_trusted_origin_create.__doc__ = """
A trusted origin is created. When an event is emitted upon the creation of a trusted origin, customers can monitor these events and take remedial action. Event is triggered when a trusted origin is created.
"""

security_trusted_origin_deactivate = Signal()
security_trusted_origin_deactivate.__doc__ = """
A trusted origin is deactivated. When an event is emitted upon the deactivation of a trusted origin, customers can monitor these events and take remedial action. Event is triggered when a trusted origin is deactivated.
"""

security_trusted_origin_delete = Signal()
security_trusted_origin_delete.__doc__ = """
A trusted origin is deleted. When an event is emitted upon the deletion of a trusted origin, customers can monitor these events and take remedial action. Event is triggered when a trusted origin is deleted.
"""

security_trusted_origin_update = Signal()
security_trusted_origin_update.__doc__ = """
A trusted origin is updated. When an event is emitted upon the modification of a trusted origin, customers can monitor these events and take remedial action. Event is triggered when a trusted origin is updated.
"""

system_api_token_create = Signal()
system_api_token_create.__doc__ = """
Create API token. This event occurs when a new unscoped API token is generated within the system. The unscoped API token grants authenticated access to the system's API for automated tasks or integration purposes. Event log details include the token ID, the user, or service it was created for, and the time of creation. This information helps maintain a secure API access framework by allowing administrators to track token issuance. Administrators can also enforce least privilege access and promptly identify any unauthorized token creation.
"""

system_api_token_request_outside_allowed_range = Signal()
system_api_token_request_outside_allowed_range.__doc__ = """
Request with API tokens made from outside the allowed network zone. Use to detect when an API token comes from an IP address that's outside of the specified allowed zone. Fired when an API token comes from an IP address that's outside of the specified allowed zone of the token.
"""

system_api_token_revoke = Signal()
system_api_token_revoke.__doc__ = """
Revoke API token.
"""

system_email_delivery = Signal()
system_email_delivery.__doc__ = """
An email's delivery status was updated. Used to notify admins of a bounced or dropped email. For certain bounce events, the context information may be lost by the email provider(s) due to email server communication delays. Such delayed bounce events will not appear in syslog. As of the 2022.08.0 release, this is also used to identify other email events e.g. delivered, deferred. See the event debugData for help identifying a remediation, such as updating an incorrect email address.
"""

system_idp_key_create = Signal()
system_idp_key_create.__doc__ = """
Identity provider key credential created. This can be used to audit that a new identity provider key credential has been created. When fired, this event indicates a new X.509 certificate credential is added to the IdP key store.
"""

system_idp_key_delete = Signal()
system_idp_key_delete.__doc__ = """
Identity provider key credential deleted. This can be used to audit that an identity provider key credential has been deleted. When fired, this event indicates a X.509 certificate credential by kid is deleted if it isn't currently being used by an active or inactive IdP.
"""

system_idp_key_update = Signal()
system_idp_key_update.__doc__ = """
Identity provider key credential updated. This can be used to audit that an identity provider key credential has been updated. When fired, this event indicates a X.509 certificate credential is updated in the IdP key store.
"""

system_idp_lifecycle_activate = Signal()
system_idp_lifecycle_activate.__doc__ = """
Identity provider activated. This can be used to audit that an identity provider has been activated. When fired, this event indicates an Identity provider was activated. This event also indicates the type of the identity provider that was activated.
"""

system_idp_lifecycle_create = Signal()
system_idp_lifecycle_create.__doc__ = """
Identity provider created. This can be used to audit that a new identity provider has been created. When fired, this event indicates an Identity provider was successfully created. This event also indicates the type of the identity provider that was created.
"""

system_idp_lifecycle_deactivate = Signal()
system_idp_lifecycle_deactivate.__doc__ = """
Identity provider deactivated. This can be used to audit that an identity provider has been deactivated. When fired, this event indicates an Identity provider has been deactivated. This event also indicates the type of the identity provider that was deactivated.
"""

system_idp_lifecycle_delete = Signal()
system_idp_lifecycle_delete.__doc__ = """
Identity provider deleted. This can be used to audit that an identity provider has been deleted. When fired, this event indicates an Identity provider was deleted. This event also indicates the type of the identity provider that was deleted.
"""

system_idp_lifecycle_read_client_secret = Signal()
system_idp_lifecycle_read_client_secret.__doc__ = """
Identity provider(s) with a client secret is read. This can be used to audit that identity provider(s) with a client secret has been read. When fired, this event indicates one or more Identity providers with a client secret was read.
"""

system_idp_lifecycle_update = Signal()
system_idp_lifecycle_update.__doc__ = """
Identity provider updated. This can be used to audit that an identity provider configuration has been updated. When fired, this event indicates an Identity provider configuration was updated. This event also indicates the type of the identity provider that was updated.
"""

system_import_complete = Signal()
system_import_complete.__doc__ = """
Import process complete.
"""

system_import_group_create = Signal()
system_import_group_create.__doc__ = """
Create group triggered by import process.
"""

system_import_group_delete = Signal()
system_import_group_delete.__doc__ = """
Remove group triggered by import process.
"""

system_import_roadblock = Signal()
system_import_roadblock.__doc__ = """
Import roadblock triggered due to exceeded threshold.
"""

system_import_start = Signal()
system_import_start.__doc__ = """
import started.
"""

system_log_stream_lifecycle_activate = Signal()
system_log_stream_lifecycle_activate.__doc__ = """
Log stream activated. This event can be used to track and audit when a user activates a log stream. When fired, this event indicates that a user activated a log stream configuration.
"""

system_log_stream_lifecycle_create = Signal()
system_log_stream_lifecycle_create.__doc__ = """
Log stream created. This event can be used to track and audit when a user creates a log stream. When fired, this event indicates that a user created a log stream configuration.
"""

system_log_stream_lifecycle_deactivate = Signal()
system_log_stream_lifecycle_deactivate.__doc__ = """
Log stream deactivated. This event can be used to track and audit when a user or Okta deactivates a log stream. When fired, this event indicates that a user or Okta deactivated a log stream configuration.
"""

system_log_stream_lifecycle_delete = Signal()
system_log_stream_lifecycle_delete.__doc__ = """
Log stream deleted. This event can be used to track and audit when a user deletes a log stream. When fired, this event indicates that a user deleted a log stream configuration.
"""

system_log_stream_lifecycle_update = Signal()
system_log_stream_lifecycle_update.__doc__ = """
Log stream updated. This event can be used to track and audit when a user updates a log stream. When fired, this event indicates that a user updated a log stream configuration.
"""

system_oauth2_token_request_outside_allowed_range = Signal()
system_oauth2_token_request_outside_allowed_range.__doc__ = """
Request with valid bearer tokens made from outside the allowed network zone. Use to detect when a bearer token comes from an IP address that's outside of the specified allowed zone. Fired when a bearer token comes from an IP address that's outside of the specified allowed zone of the client.
"""

system_org_rate_limit_violation = Signal()
system_org_rate_limit_violation.__doc__ = """
Rate limit violation.
"""

system_org_rate_limit_warning = Signal()
system_org_rate_limit_warning.__doc__ = """
Rate limit warning.
"""

system_sms_send_phone_verification_message = Signal()
system_sms_send_phone_verification_message.__doc__ = """
Send phone verification SMS message. As of the 2022.06.0 release this event is also used to identify transactions blocked by Okta, which is indicated by a "deny" outcome. Previously, the system.operation.rate_limit.violation was used to identify blocked transactions. Additionally, the method of generating the MobilePhone ID in the event has changed for Okta Classic. It has not changed for Okta Identity Engine.
"""

system_voice_send_phone_verification_call = Signal()
system_voice_send_phone_verification_call.__doc__ = """
Send phone verification call. As of the 2022.06.0 release this event is also used to identify transactions blocked by Okta, which is indicated by a "deny" outcome. Previously, the system.operation.rate_limit.violation was used to identify blocked transactions. Additionally, the method of generating the MobilePhone ID in the event has changed for Okta Classic. It has not changed for Okta Identity Engine.
"""

user_account_lock = Signal()
user_account_lock.__doc__ = """
Auto-lock user account for Okta.
"""

user_account_privilege_grant = Signal()
user_account_privilege_grant.__doc__ = """
A User's admin privileges changed. This can be used to audit the provisioning of admin privileges for users. When fired, this event contains information about the type of admin privileges the user currently has.  The list of current privileges contain both individually assigned roles  as well as the ones granted to the user through their group membership. Related events include: USER_ACCOUNT_PRIVILEGE_REVOKE.
"""

user_account_privilege_revoke = Signal()
user_account_privilege_revoke.__doc__ = """
All of user's admin privilege revoked. This can be used to audit the deprovisioning of admin privileges from users. When fired, this event indicates the user has no more admin privileges. All of user's privileges were revoked including individually assigned roles  as well as the ones granted to the user through their group membership. Related events include: USER_ACCOUNT_PRIVILEGE_GRANT.
"""

user_account_report_suspicious_activity_by_enduser = Signal()
user_account_report_suspicious_activity_by_enduser.__doc__ = """
User reported suspicious activity. This event is used to identify user account suspicious activity.
"""

user_account_reset_password = Signal()
user_account_reset_password.__doc__ = """
User's Okta password has been reset.
"""

user_account_unlock = Signal()
user_account_unlock.__doc__ = """
Auto-unlock user account for Okta.
"""

user_account_unlock_by_admin = Signal()
user_account_unlock_by_admin.__doc__ = """
User account unlock by admin.
"""

user_account_update_password = Signal()
user_account_update_password.__doc__ = """
User update password for Okta.
"""

user_account_update_profile = Signal()
user_account_update_profile.__doc__ = """
Update user profile for Okta.
"""

user_authentication_auth_via_IDP = Signal()
user_authentication_auth_via_IDP.__doc__ = """
Authenticate user via IDP.
"""

user_authentication_auth_via_mfa = Signal()
user_authentication_auth_via_mfa.__doc__ = """
Authentication of user via MFA. For Okta Classic orgs, this event will only fire for second factor verifications, whereas for Identity Engine orgs, this event will fire for both primary and second factor verifications.
"""

user_authentication_auth_via_social = Signal()
user_authentication_auth_via_social.__doc__ = """
Authenticate user with social login.
"""

user_authentication_authenticate = Signal()
user_authentication_authenticate.__doc__ = """
Authentication via device trust certificate.
"""

user_authentication_sso = Signal()
user_authentication_sso.__doc__ = """
Fired when a user performs a single sign-on (SSO) to an app instance and contains the client details of the user. Can be used to identify when a user attempted to sign into an application for audit or debugging purposes. Note that the event is fired even when the sign-on is unsuccessful.
"""

user_authentication_universal_logout = Signal()
user_authentication_universal_logout.__doc__ = """
This event is fired when an admin or system account triggers Universal Logout against an app instance. It contains the app instance details for which the Universal Logout API was fired. This event identifies when applications have had Universal Logout triggered for audit or debugging purposes. This event is only fired once. It's only fired for applications that have been configured for Universal Logout. You can configure it under Risk policy, Post Auth Session policy, or in an admin-initiated Clear User Session.
"""

user_authentication_universal_logout_scheduled = Signal()
user_authentication_universal_logout_scheduled.__doc__ = """
This event is fired when an admin manually triggers Universal Logout for a user. It contains context about the initiating request, such as where the request originated and how the Universal Logout endpoint was invoked. After Universal Logout is complete, the user.authentication.universal_logout event is fired, and you can correlate both events using the traceID. This event identifies the geolocation, IP address, and IP chain of the requesting entity. This event is only fired once. You can correlate this event with the user.authentication.universal_logout event using traceID.
"""

user_behavior_profile_reset = Signal()
user_behavior_profile_reset.__doc__ = """
User behavior profile reset. This event can be used to identify resets to a user behavior profiles, which may be helpful when troubleshooting unexpected behavior detection evaluations. This event is triggered when an administrator manually resets a user's behavior profile in the Admin Console.
"""

user_credential_enroll = Signal()
user_credential_enroll.__doc__ = """
Device Trust certificate enrollment.
"""

user_identity_verification = Signal()
user_identity_verification.__doc__ = """
This event is fired when a user is directed to complete an Identity Verification as a result of an Okta Account Management (OAMP) Policy evaluation. This event indicates an identity verification request has occurred and will contain the results of the id verification. Completion of an id verification will determine whether the trigger OAMP operation can be completed; start is recorded by user.identity_verification.start.
"""

user_identity_verification_start = Signal()
user_identity_verification_start.__doc__ = """
An Okta account management policy (OAMP) prompted the user to verify their identity with an identity verification service. Helps admins audit identity-verification prompts and troubleshoot IDV-flow issues. Initiates the prompt for identity-verification flow; completion is recorded by user.identity_verification event.
"""

user_import_password = Signal()
user_import_password.__doc__ = """
Imported user password from external system during login. This can be used to understand if a user password import attempt was successful or if it failed. If the attempt failed, the password import will be tried again on a subsequent successful login. When fired, this event contains information about the import type, and whether or not the password import was successful. If the import is successful, it is safe to "clean up" that user from an external system. If the import failed, Okta will continue retrying the import during every successful authentication attempt until the password is successfully imported. Check the failure reason for details about whether any action is needed for the import to succeed.
"""

user_lifecycle_activate = Signal()
user_lifecycle_activate.__doc__ = """
Activate Okta user.
"""

user_lifecycle_create = Signal()
user_lifecycle_create.__doc__ = """
Create Okta user.
"""

user_lifecycle_deactivate = Signal()
user_lifecycle_deactivate.__doc__ = """
Deactivate Okta user.
"""

user_lifecycle_delete_initiated = Signal()
user_lifecycle_delete_initiated.__doc__ = """
Delete Okta user initiated.
"""

user_lifecycle_reactivate = Signal()
user_lifecycle_reactivate.__doc__ = """
Reactivate Okta user.
"""

user_lifecycle_suspend = Signal()
user_lifecycle_suspend.__doc__ = """
Suspend Okta user.
"""

user_lifecycle_unsuspend = Signal()
user_lifecycle_unsuspend.__doc__ = """
Unsuspend Okta user.
"""

user_mfa_factor_activate = Signal()
user_mfa_factor_activate.__doc__ = """
Activate factor or authenticator enrollment method for user. Provides org admins with audit log and oversight utility for an MFA factor when it is activated. When fired, the event contains information about the MFA factor that has been activated, as well as the target user and the user activating the factor. For Identity Engine orgs, this event will fire when an authentication method is enrolled.
"""

user_mfa_factor_deactivate = Signal()
user_mfa_factor_deactivate.__doc__ = """
Reset factor or authenticator enrollment method for user. Provides org admins with audit log and oversight utility for the change in MFA factor lifecycle status when a specific factor is permanently deactivated. When fired, the event contains information about the MFA factor that has been deactivated, as well as the target user and the user deactivating the factor. For Identity Engine orgs, this event will fire when an authentication method is unenrolled.
"""

user_mfa_factor_reset_all = Signal()
user_mfa_factor_reset_all.__doc__ = """
Reset all factors or authenticator enrollments for user. Provides org admins with audit log and oversight utility for the change in MFA factor lifecycle statuses when all MFA factors for a user are permanently deactivated. When fired, the event contains information about the target user for whom all factors have been deactivated, as well as the user resetting the factors. For Identity Engine orgs, this event contains information about a target user for whom all authenticator enrollments have been reset.
"""

user_mfa_factor_suspend = Signal()
user_mfa_factor_suspend.__doc__ = """
Suspend factor or authenticator enrollment method for user. Provides org admins with audit log and oversight utility for the change in MFA factor lifecycle status when a factor is suspended, usually as a result of suspected compromise. When fired, the event contains information about the MFA factor that has been suspended, as well as the target user and the user suspending the factor. When unsuspended, related event user.mfa.factor.unsuspend will be fired.
"""

user_mfa_factor_unsuspend = Signal()
user_mfa_factor_unsuspend.__doc__ = """
Unsuspend factor or authenticator enrollment method for user. Provides org admins with audit log and oversight utility for the change in MFA factor lifecycle status when a factor is reactivated from a state of suspension, after it has been determined that the authenticator is secure. When fired, the event contains information about the MFA factor that has been unsuspended, as well as the target user and the user reactivating the suspended factor. Before suspension, related event user.mfa.factor.suspend would have been fired.
"""

user_mfa_factor_update = Signal()
user_mfa_factor_update.__doc__ = """
Update factor for user.
"""

user_risk_change = Signal()
user_risk_change.__doc__ = """
Indicates a user's risk level has changed. This event can be used to monitor risk level changes for users. This event triggers when Okta determines that a user is associated with a change in risk activity or context.
"""

user_risk_detect = Signal()
user_risk_detect.__doc__ = """
Indicates a user risk was detected. This event can be used to monitor risk level detections for users. This event triggers when Okta detects that a user is associated with risk activity or context.
"""

user_session_clear = Signal()
user_session_clear.__doc__ = """
Clear user session.
"""

user_session_context_change = Signal()
user_session_context_change.__doc__ = """
User session context changed. This event indicates that the context in which the session is being used has changed significantly enough from the context in which the event was created, that re-evaluation of policy may be required. Often this indicates a security issue related to the session.
"""

user_session_end = Signal()
user_session_end.__doc__ = """
User logout from Okta.
"""

user_session_start = Signal()
user_session_start.__doc__ = """
User login to Okta.
"""



"""
Here are some utility pieces.
"""

from logging import getLogger

from django.conf import settings

from asgiref.sync import async_to_sync
from okta.client import Client as OktaClient

LOGGER = getLogger(__name__)

class OktaAPIClient:
	"""
	Handles interactions with the Okta API, including lazy instantiation of the Okta client and making API requests.
	"""

	def __getattr__(self, name):
		"""Lazy instantiation
		It provides a mechanism for lazy instantiation of the Okta API client, its credentials, and the Okta organization URL.

		:param name: The name of the attribute being accessed.
		:type name: str
		:returns: the attribute value
		"""

		if name == 'okta_api_client':
			client_config = {'orgUrl': settings.OKTA_CLIENT['ORG_URL']} | self.okta_api_credentials
			if 'SSL_CONTEXT' in settings.OKTA_CLIENT:
				client_config['sslContext'] = settings.OKTA_CLIENT['SSL_CONTEXT']
			value = OktaClient(client_config)
		elif name == 'okta_api_credentials':
			if ('API_CLIENT_ID' in settings.OKTA_CLIENT) and ('API_PRIVATE_KEY' in settings.OKTA_CLIENT):
				value = {
					'authorizationMode'	: 'PrivateKey',
					'clientId'			: settings.OKTA_CLIENT['API_CLIENT_ID'],
					'privateKey'		: settings.OKTA_CLIENT['API_PRIVATE_KEY'],
					'scopes'			: settings.OKTA_CLIENT.get('API_SCOPES', None),
				}
			elif 'API_TOKEN' in settings.OKTA_CLIENT:
				value = {'token': settings.OKTA_CLIENT['API_TOKEN']}
			else:
				raise RuntimeError('Missing auth settings for Okta client')
		else:
			return getattr(super(), name)
		self.__setattr__(name, value)
		return value

	def __call__(self, method_name, *args, **kwargs):
		"""Okta API request
		Makes a request to the Okta API using the configured Okta client.

		:param method_name: The name of the method to call on the Okta client (e.g., 'list_users').
		:type method_name: str
		:param args: Positional arguments to pass to the Okta client method.
		:param kwargs: Keyword arguments to pass to the Okta client method.
		:return: The result of the Okta API call.
		"""

		result = async_to_sync(getattr(self.okta_api_client, method_name), )(*args, **kwargs)

		if len(result) == 3:
			result, response, err = result
		elif len(result) == 2:
			response, err = result
		else:
			raise RuntimeError('Unknown result: {}'.format(result))

		if err is not None:
			raise RuntimeError(err)

		while response.has_next():
			partial, err = async_to_sync(response.next)()
			if err is not None:
				raise RuntimeError(err)
			result.extend(partial)

		return result



#! python
"""Okta client Django app
This app adapts your Django site as a service provider to work with Okta SAML authentication.

It defines a custom AUTH_USER_MODEL based on the Okta API User object and it has an optional authentication backend that connects to the Okta API and synchronizes the users and groups on login.

There's also a mixin to handle Okta Event Hooks in your views easily.

TODO:
- Everything

Refs:
- ?
"""

__version__ = '0.5.5'



#python
"""
URL patterns for the Okta client application.
"""

from django.conf import settings
from django.urls import path, include
from django.views.decorators.csrf import csrf_exempt

from . import views

app_name = 'okta-client'

if hasattr(settings, 'OKTA_CLIENT'):
	urlpatterns = [
		path('accounts/login/', views.LoginView.as_view(), name='login'),
		path('accounts/logout/', views.LogoutView.as_view(), name='logout'),
		path('/'.join((settings.OKTA_CLIENT['LOCAL_PATH'], 'acs/')), csrf_exempt(views.ACSView.as_view()), name='acs'),
		path('/'.join((settings.OKTA_CLIENT['LOCAL_PATH'], 'event_hooks/')), views.OktaEventHooks.as_view(), name='event_hooks'),
	]
else:
	urlpatterns = [
		path('accounts/', include('django.contrib.auth.urls')),
	]



#python
"""
Django Okta Client Views
"""

from json import JSONDecodeError
from logging import getLogger

from django.contrib.auth import get_user_model
from django.contrib.auth.mixins import LoginRequiredMixin
from django.http import HttpResponse, HttpResponseBadRequest, HttpResponseRedirect, JsonResponse
from django.shortcuts import render
from django.views import View

from rest_framework.permissions import IsAuthenticated as RESTIsAuthenticated
from rest_framework.views import APIView

from .exceptions import SAMLAssertionError
from .mixins import LoginLogoutMixin, OktaEventHookMixin

LOGGER = getLogger(__name__)


class ACSView(LoginLogoutMixin, View):
	"""
	Handles the Assertion Consumer Service (ACS) for SAML responses from Okta.
	"""

	def post(self, request):
		"""POST verb
		Handles the POST request for the ACS endpoint, processing the SAML assertion.

		:param request: the Django request
		:type request: object
		:returns object: the Django response
		"""

		try:
			next_url = self.saml_assertion(request)
		except SAMLAssertionError:
			return HttpResponseBadRequest()
		else:
			return HttpResponseRedirect(next_url)


class LoginView(LoginLogoutMixin, View):
	"""
	Handles the login process for Okta.
	"""

	def get(self, request):
		"""GET verb
		Handles the GET request for the login endpoint.

		:param request: the Django request
		:type request: object
		:returns object: the Django response
		"""

		try:
			next_url = self.login_user(request)
		except RuntimeError:
			return HttpResponseBadRequest()
		else:
			return HttpResponseRedirect(next_url)


class LogoutView(LoginLogoutMixin, LoginRequiredMixin, View):
	"""
	Handles the logout process for Okta.
	"""

	def get(self, request):
		"""GET verb
		Handles the GET request for the logout endpoint.

		:param request: the Django request
		:type request: object
		:returns object: the Django response
		"""

		next_url = self.logout_user(request)
		return HttpResponseRedirect(next_url)

	def post(self, request):
		"""POST verb
		Handles the POST request for the logout endpoint.

		:param request: the Django request
		:type request: object
		:returns object: the Django response
		"""

		next_url = self.logout_user(request)
		return HttpResponseRedirect(next_url)


class OktaEventHooks(OktaEventHookMixin, APIView):
	"""
	Handles Okta event hooks.
	"""

	permission_classes = [RESTIsAuthenticated]

	def get(self, request):
		"""HTTP GET
		Only used to confirm that it follows Okta's convention.

		:param request: the Django request
		:type request: object
		:returns object: the Django response
		"""

		return JsonResponse(self.authenticate_endpoint(request))

	def post(self, request):
		"""HTTP GET
		Regular Event Hook handling.

		:param request: the Django request
		:type request: object
		:returns object: the Django response
		"""

		try:
			self.handle_events(request)
		except (JSONDecodeError, UnicodeDecodeError):
			return HttpResponseBadRequest()
		else:
			return HttpResponse(status=204)


######## Demo View ########

class IndexView(LoginRequiredMixin, View):
	"""
	A simple view to demonstrate a protected page after successful login.
	"""

	def get(self, request):
		"""GET verb
		Handles the GET request for the index page.

		:param request: the Django request
		:type request: object
		:returns object: the Django response
		"""

		return render(request, 'okta-client/index.html')


#python
"""
Django application configuration for the Okta client.
"""

from logging import getLogger

from django.apps import AppConfig

from .signals.events import user_lifecycle_create

LOGGER = getLogger(__name__)


class OktaClientConfig(AppConfig):
	"""
	Configuration class for the 'okta_client' Django application.
	"""

	default_auto_field = 'django.db.models.BigAutoField'
	name = 'okta_client'



#python
"""
Okta Client Django settings.
"""

from logging import getLogger
from urllib.parse import urlsplit, urlunsplit

from normalized_django_settings import decode_setting, path_for_setting, setting_is_true

DEFAULT_LOCAL_PATH = 'okta_client'
LOGGER = getLogger(__name__)

EXPECTED_VALUES_FROM_ENV = {
	'OKTA_CLIENT_OAUTH_SETTINGS_FROM_ENV': {
		'OKTA_CLIENT_ID',
		'OKTA_CLIENT_SCOPES',
	},
	'OKTA_CLIENT_SETTINGS_FROM_ENV' : {
		'OKTA_CLIENT_LOCAL_PATH',
		'OKTA_CLIENT_ORG_URL',
		'OKTA_CLIENT_PRIVATE_KEY',
		'OKTA_CLIENT_TOKEN',
		'OKTA_DJANGO_STAFF_USER_GROUPS',
		'OKTA_DJANGO_SUPER_USER_GROUPS',
		'OKTA_SAML_ASSERTION_DOMAIN_URL',
		'OKTA_SAML_METADATA_AUTO_CONF_URL',
	},
}

IMPLICIT_ENVIRONMENTAL_SETTINGS = {
  'OKTA_CLIENT_LOCAL_PATH' : DEFAULT_LOCAL_PATH,
}

def normalized_settings(**django_settings):
	"""Common values for Django
	Applies common Okta client settings to a Django settings dictionary.

	:param django_settings: the current globals() in Django's site
	:type django_settings: Any
	:return: new content for "globals"
	"""

	if 'okta_client' not in django_settings['INSTALLED_APPS']:
		django_settings['INSTALLED_APPS'].append('okta_client')

	if 'AUTH_USER_MODEL' not in django_settings:
		django_settings['AUTH_USER_MODEL'] = 'okta_client.OktaUser'

	okta_client = {}
	if 'OKTA_SAML_METADATA_AUTO_CONF_URL' in django_settings['ENVIRONMENTAL_SETTINGS_KEYS']:
		okta_client |= {
			'METADATA_AUTO_CONF_URL': django_settings['ENVIRONMENTAL_SETTINGS']['OKTA_SAML_METADATA_AUTO_CONF_URL'],
			'ORG_URL': urlunsplit(urlsplit(django_settings['ENVIRONMENTAL_SETTINGS']['OKTA_SAML_METADATA_AUTO_CONF_URL'])[:2] + ('', '', '')),
		}
		if 'OKTA_SAML_ASSERTION_DOMAIN_URL' in django_settings['ENVIRONMENTAL_SETTINGS']:
			okta_client['ASSERTION_DOMAIN_URL'] = django_settings['ENVIRONMENTAL_SETTINGS']['OKTA_SAML_ASSERTION_DOMAIN_URL']
	elif 'OKTA_CLIENT_ORG_URL' in django_settings['ENVIRONMENTAL_SETTINGS_KEYS']:
		okta_client['ORG_URL'] = django_settings['ENVIRONMENTAL_SETTINGS']['OKTA_CLIENT_ORG_URL']

	okta_api_client_key = decode_setting(django_settings, 'OKTA_CLIENT_PRIVATE_KEY')
	if (okta_api_client_key is not None) and not isinstance(okta_api_client_key, str):
		okta_api_client_key = okta_api_client_key.decode('utf-8')
	okta_api_client = None
	if (EXPECTED_VALUES_FROM_ENV['OKTA_CLIENT_OAUTH_SETTINGS_FROM_ENV'].issubset(django_settings['ENVIRONMENTAL_SETTINGS_KEYS'])) and (okta_api_client_key is not None):
		okta_api_client = {
			'API_CLIENT_ID': django_settings['ENVIRONMENTAL_SETTINGS']['OKTA_CLIENT_ID'],
			'API_SCOPES': django_settings['ENVIRONMENTAL_SETTINGS']['OKTA_CLIENT_SCOPES'].split(','),
			'API_PRIVATE_KEY': okta_api_client_key
		}
	elif 'OKTA_CLIENT_TOKEN' in django_settings['ENVIRONMENTAL_SETTINGS_KEYS']:
		okta_api_client = {'API_TOKEN': django_settings['ENVIRONMENTAL_SETTINGS']['OKTA_CLIENT_TOKEN']}
	else:
		LOGGER.debug('The Okta API client is not configured')

	if okta_api_client is not None:

		okta_client |= okta_api_client

		if 'OKTA_DJANGO_SUPER_USER_GROUPS' in django_settings['ENVIRONMENTAL_SETTINGS_KEYS']:
			okta_client['SUPER_USER_GROUPS'] = [group.strip() for group in django_settings['ENVIRONMENTAL_SETTINGS']['OKTA_DJANGO_SUPER_USER_GROUPS'].split(',')]
		if 'OKTA_DJANGO_STAFF_USER_GROUPS' in django_settings['ENVIRONMENTAL_SETTINGS_KEYS']:
			okta_client['STAFF_USER_GROUPS'] = [group.strip() for group in django_settings['ENVIRONMENTAL_SETTINGS']['OKTA_DJANGO_STAFF_USER_GROUPS'].split(',')]

	if okta_client:
		django_settings['OKTA_CLIENT'] = okta_client
		django_settings['OKTA_CLIENT']['LOCAL_PATH'] = django_settings['ENVIRONMENTAL_SETTINGS']['OKTA_CLIENT_LOCAL_PATH']

		if 'AUTHENTICATION_BACKENDS' not in django_settings:
			django_settings['AUTHENTICATION_BACKENDS'] = ['okta_client.auth_backends.OktaBackend', 'django.contrib.auth.backends.ModelBackend']
		elif 'okta_client.auth_backends.OktaBackend' not in django_settings['AUTHENTICATION_BACKENDS']:
			django_settings['AUTHENTICATION_BACKENDS'] = ['okta_client.auth_backends.OktaBackend'] + django_settings['AUTHENTICATION_BACKENDS']

	if 'REST_FRAMEWORK' not in django_settings:
		django_settings['REST_FRAMEWORK'] = {
			'DEFAULT_AUTHENTICATION_CLASSES': ['okta_client.auth_backends.DjangoRESTBearerTokenAuthentication', 'rest_framework.authentication.SessionAuthentication'],
		}
		if 'rest_framework' not in django_settings['INSTALLED_APPS']:
			django_settings['INSTALLED_APPS'].append('rest_framework')
		if 'rest_framework.authtoken' not in django_settings['INSTALLED_APPS']:
			django_settings['INSTALLED_APPS'].append('rest_framework.authtoken')

	return django_settings
